
/***************************************************************************
 * IPv4Header.cc -- The IPv4Header Class represents an IPv4 datagram. It   *
 * contains methods to set any header field. In general, these methods do  *
 * error checkings and byte order conversion.                              *
 *                                                                         *
 ***********************IMPORTANT NMAP LICENSE TERMS************************
 *                                                                         *
 * The Nmap Security Scanner is (C) 1996-2009 Insecure.Com LLC. Nmap is    *
 * also a registered trademark of Insecure.Com LLC.  This program is free  *
 * software; you may redistribute and/or modify it under the terms of the  *
 * GNU General Public License as published by the Free Software            *
 * Foundation; Version 2 with the clarifications and exceptions described  *
 * below.  This guarantees your right to use, modify, and redistribute     *
 * this software under certain conditions.  If you wish to embed Nmap      *
 * technology into proprietary software, we sell alternative licenses      *
 * (contact sales@insecure.com).  Dozens of software vendors already       *
 * license Nmap technology such as host discovery, port scanning, OS       *
 * detection, and version detection.                                       *
 *                                                                         *
 * Note that the GPL places important restrictions on "derived works", yet *
 * it does not provide a detailed definition of that term.  To avoid       *
 * misunderstandings, we consider an application to constitute a           *
 * "derivative work" for the purpose of this license if it does any of the *
 * following:                                                              *
 * o Integrates source code from Nmap                                      *
 * o Reads or includes Nmap copyrighted data files, such as                *
 *   nmap-os-db or nmap-service-probes.                                    *
 * o Executes Nmap and parses the results (as opposed to typical shell or  *
 *   execution-menu apps, which simply display raw Nmap output and so are  *
 *   not derivative works.)                                                *
 * o Integrates/includes/aggregates Nmap into a proprietary executable     *
 *   installer, such as those produced by InstallShield.                   *
 * o Links to a library or executes a program that does any of the above   *
 *                                                                         *
 * The term "Nmap" should be taken to also include any portions or derived *
 * works of Nmap.  This list is not exclusive, but is meant to clarify our *
 * interpretation of derived works with some common examples.  Our         *
 * interpretation applies only to Nmap--we don't speak for other people's  *
 * GPL works.                                                              *
 *                                                                         *
 * If you have any questions about the GPL licensing restrictions on using *
 * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
 * we also offer alternative license to integrate Nmap into proprietary    *
 * applications and appliances.  These contracts have been sold to dozens  *
 * of software vendors, and generally include a perpetual license as well  *
 * as providing for priority support and updates as well as helping to     *
 * fund the continued development of Nmap technology.  Please email        *
 * sales@insecure.com for further information.                             *
 *                                                                         *
 * As a special exception to the GPL terms, Insecure.Com LLC grants        *
 * permission to link the code of this program with any version of the     *
 * OpenSSL library which is distributed under a license identical to that  *
 * listed in the included COPYING.OpenSSL file, and distribute linked      *
 * combinations including the two. You must obey the GNU GPL in all        *
 * respects for all of the code used other than OpenSSL.  If you modify    *
 * this file, you may extend this exception to your version of the file,   *
 * but you are not obligated to do so.                                     *
 *                                                                         *
 * If you received these files with a written license agreement or         *
 * contract stating terms other than the terms above, then that            *
 * alternative license agreement takes precedence over these comments.     *
 *                                                                         *
 * Source is provided to this software because we believe users have a     *
 * right to know exactly what a program is going to do before they run it. *
 * This also allows you to audit the software for security holes (none     *
 * have been found so far).                                                *
 *                                                                         *
 * Source code also allows you to port Nmap to new platforms, fix bugs,    *
 * and add new features.  You are highly encouraged to send your changes   *
 * to nmap-dev@insecure.org for possible incorporation into the main       *
 * distribution.  By sending these changes to Fyodor or one of the         *
 * Insecure.Org development mailing lists, it is assumed that you are      *
 * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
 * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
 * will always be available Open Source, but this is important because the *
 * inability to relicense code has caused devastating problems for other   *
 * Free Software projects (such as KDE and NASM).  We also occasionally    *
 * relicense the code to third parties as discussed above.  If you wish to *
 * specify special license conditions of your contributions, just say so   *
 * when you send them.                                                     *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License v2.0 for more details at                         *
 * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
 * included with Nmap.                                                     *
 *                                                                         *
 ***************************************************************************
 * The code in this file was originally written by Luis MartinGarcia       *
 * during Google Summer of Code programme, 2009.                           *
 *                                                                         *
 ***************************************************************************/


#include "IPv4Header.h"
#include "nping.h"
#include "common.h"
#include "dnet.h"
#include "utils_net.h"

//extern NpingOps o;



IPv4Header::IPv4Header() {

  memset(&h, 0, 20 + MAX_IP_OPTIONS_LEN);

  ipoptlen=0;
  length=20;   /* Initial value 20. This will be incremented if options are
                * used */

  setVersion();
  setHeaderLength();
  setTTL(DEFAULT_IP_TTL);
  setTotalLength();

} /* End of IPv4Header constructor */

IPv4Header::~IPv4Header() {


} /* End of IPv4Header destructor */


/** Sets every class attribute to zero */
void IPv4Header::zero(){

      memset(&h, 0, 20 + MAX_IP_OPTIONS_LEN);
      ipoptlen=0;
      length=0;

} /* End of zero() */




/** @warning This method is essential for the superclass getBinaryBuffer()
 *  method to work. Do NOT change a thing unless you know what you're doing  */
u8 *IPv4Header::getBufferPointer(){

  return (u8*)(&h);

} /* End of getBufferPointer() */


int IPv4Header::setVersion(){

  h.ip_v   = 4;

  return 4;

} /* End of setVersion() */


u8 IPv4Header::getVersion(){

  return (u8)4;
} /* End of getVersion() */


bool IPv4Header::validateVersion(u8 ver){

    if( ver == 4 )
        return true;
    else
        return false;

} /* End of validateVersion() */


bool IPv4Header::validateVersion(){

    return validateVersion( this->getVersion() );

} /* End of validateVersion() */




int IPv4Header::setHeaderLength(){

  h.ip_hl  = 5 + (ipoptlen/4);

  return OP_SUCCESS;
} /* End of setHeaderLength() */



int IPv4Header::setHeaderLength(u8 l){

  h.ip_hl  = l;

  return OP_SUCCESS;
} /* End of setHeaderLength() */


u8 IPv4Header::getHeaderLength(){

  return h.ip_hl;

} /* End of getHeaderLength() */


bool IPv4Header::validateHeaderLength(u8 len){

    if( len >= 5 && len <= (60/4) )
        return true;
    else
        return false;

} /* End of validateHeaderLength() */



bool IPv4Header::validateHeaderLength(){

    return validateHeaderLength( this->getHeaderLength() );

} /* End of validateHeaderLength() */




int IPv4Header::setTOS(u8 v){

  h.ip_tos = v;

  return OP_SUCCESS;

} /* End of setTOS() */


u8 IPv4Header::getTOS(){

  return h.ip_tos;

} /* End of getTOS() */


bool IPv4Header::validateTOS(u8 t){

    if( 1 ) /* TODO: Check http://www.iana.org/assignments/ip-parameters */
        return true;
    else
        return false;

} /* End of validateHeaderLength() */



bool IPv4Header::validateTOS(){

    return validateTOS( this->getTOS() );

} /* End of validateHeaderLength() */





int IPv4Header::setTotalLength(){

  int mylen = 4*getHeaderLength();
  int otherslen=0;

  if (next!=NULL)
      otherslen=next->getLen();

  h.ip_len=htons( mylen+otherslen );

  return OP_SUCCESS;
} /* End of setTotalLength() */





/** @warning Supplied value MUST be in host byte order because it will get
 *  converted by this method using htons() */
int IPv4Header::setTotalLength(u16 l){

  h.ip_len = htons(l);

  return OP_SUCCESS;
} /* End of setTotalLength() */


/** @warning Returned value is already in host byte order. */
u16 IPv4Header::getTotalLength(){

  return ntohs(h.ip_len);

} /* End of getTotalLength() */



bool IPv4Header::validateTotalLength(u16 t){

    if( t >= 20 ) /* TODO: Ideally we should check if supplied lenght is
                   * greater that header lenght (it may contain options) */
        return true;
    else
        return false;

} /* End of validateHeaderLength() */



bool IPv4Header::validateTotalLength(){

    if( getTotalLength() > 20 + ipoptlen ) /* TODO: test this */
        return true;
    else
        return false;

} /* End of validateHeaderLength() */




/** Sets identification field to a random value */
int IPv4Header::setIdentification(){

  h.ip_id=get_random_u16();

  return OP_SUCCESS;
} /* End of setIdentification() */


/** @warning Supplied value MUST be in host byte order because it will get
 *  converted by this method using htons() */
int IPv4Header::setIdentification(u16 i){

  h.ip_id = htons(i);

  return OP_SUCCESS;
} /* End of setIdentification() */







/** @warning Returned value is already in host byte order. */
u16 IPv4Header::getIdentification(){

  return ntohs(h.ip_id);
} /* End of getIdentification() */



bool IPv4Header::validateIdentification(u16 i){

    if( 1 ) /* ID value is always valid by itself */
        return true;
    else
        return false;

} /* End of validateHeaderLength() */




bool IPv4Header::validateIdentification(){

    return validateIdentification( this->getIdentification() );

} /* End of validateHeaderLength() */





/** Sets fragment offset field to a random value */
int IPv4Header::setFragOffset(){

  h.ip_off=get_random_u16();
    /* TODO: Should we check here that i<8192 ? */

  return OP_SUCCESS;
} /* End of setFragOffset() */


/** @warning Supplied value MUST be in host byte order because it will get
 *  converted by this method using htons() */
int IPv4Header::setFragOffset(u16 i){

  h.ip_off = htons(i);

  /* TODO: Should we check here that i<8192 ? */

  return OP_SUCCESS;
} /* End of setFragOffset() */


/** @warning Returned value is already in host byte order. */
u16 IPv4Header::getFragOffset(){

  return ntohs(h.ip_off);

} /* End of getFragOffset() */




bool IPv4Header::validateFragOffset(u16 f){

    u16 fragoff = 8 * (f & IP_OFFMASK); /* This is how it's done in nmap's
                                         * tcpip.cc::validatepkt() */
    if( !fragoff )
        return true;
    else
        return false;

} /* End of validateFragOffset() */




bool IPv4Header::validateFragOffset(){

    return validateFragOffset( this->getFragOffset() );

} /* End of validateFragOffset() */








/** Set RF flag */
int IPv4Header::setRF(){

  h.ip_off |= htons(IP_RF);

  return OP_SUCCESS;
} /* End of setRF() */

/** Unset RF flag */
int IPv4Header::unsetRF(){

  h.ip_off ^= htons(IP_RF);

  return OP_SUCCESS;
} /* End of unsetRF() */


/** Get RF flag */
bool IPv4Header::getRF(){

  return h.ip_off & htons(IP_RF);

} /* End of getRF() */


/** Checks whether the RF flag is set or not. According to RFC 791 this flag
 *  should be always set to zero so we return false if it's set to one.      */
bool IPv4Header::validateRF(){

    if ( getRF() == true )
        return false;
    else
        return true;

} /* End of validateRF() */


/** Set MF flag */
int IPv4Header::setMF(){

  h.ip_off |= htons(IP_MF);

  return OP_SUCCESS;
} /* End of setMF() */

/** Unset MF flag */
int IPv4Header::unsetMF(){

  h.ip_off ^= htons(IP_MF);

  return OP_SUCCESS;
} /* End of unsetMF() */


/* Get MF flag */
bool IPv4Header::getMF(){

  return h.ip_off & htons(IP_MF);
} /* End of getMF() */



/** Set DF flag */
int IPv4Header::setDF(){

  h.ip_off |= htons(IP_DF);

  return OP_SUCCESS;
} /* End of setDF() */

/** Unset DF flag */
int IPv4Header::unsetDF(){

  h.ip_off ^= htons(IP_DF);

  return OP_SUCCESS;
} /* End of unsetDF() */


/** Get DF flag */
bool IPv4Header::getDF(){

  return h.ip_off & htons(IP_DF);
} /* End of getDF) */







/** Returns true if the flags are set correctly. This is, the RF flag is set
 *  to zero and only one (or none) of the DF and MF flags are set.           */
bool IPv4Header::validateFlags(){

    if ( getRF() == true )
        return false;
    else if ( getMF()==true && getDF()==true )
        return false;
    else
        return true;

} /* End of validateFlags() */



/** Sets TTL field to a random value */
int IPv4Header::setTTL(){

  h.ip_ttl=get_random_u8();

  return OP_SUCCESS;
} /* End of setTTL() */


/** @warning Supplied value MUST be in host byte order because it will get
 *  converted by this method using htons() */
int IPv4Header::setTTL(u8 t){

  h.ip_ttl = t;

  return OP_SUCCESS;

} /* End of setTTL() */


/** @warning Returned value is already in host byte order. */
u8 IPv4Header::getTTL(){

  return h.ip_ttl;

} /* End of getTTL() */



/** Sets field "next protocol" to the supplied value.
 *  @warning: No error checks are made. Make sure the supplied value
 *  corresponds to an actual IANA number. Check
 *  http://www.iana.org/assignments/protocol-numbers/ for more details.      */
int IPv4Header::setNextProto(u8 p){

  h.ip_p = htons(p);

  return OP_SUCCESS;
} /* End of setNextProto() */




/** Sets field "next protocol" to the number that corresponds to the supplied
 *  protocol name. Currently onyl TCP, UDP and ICMP are supported. Any
 *  help to extend this functionality would be appreciated. For a list of all
 *  proto names and numbers check:
 *  http://www.iana.org/assignments/protocol-numbers/                        */
int IPv4Header::setNextProto(const char *p){

  if (p==NULL){
        printf("setNextProto(): NULL pointer supplied\n");
    return OP_FAILURE;
  }

  if( !strcasecmp(p, "TCP") )
        h.ip_p=6;   /* 6=IANA number for proto TCP */

  else if( !strcasecmp(p, "UDP") )
        h.ip_p=17;  /* 17=IANA number for proto UDP */

  else if( !strcasecmp(p, "ICMP") )
        h.ip_p=1;   /* 1=IANA number for proto ICMP */
  else{
        printf("setNextProto(): Invalid protocol number\n");
        return OP_FAILURE;
  }

  return OP_SUCCESS;
} /* End of setNextProto() */



/** Returns next protocol number */
u8 IPv4Header::getNextProto(){

  return h.ip_p;

} /* End of getNextProto() */


/** Assigned protocol numbers range from 0 to 140 (both included) as of
 *  Jun 11 2009. Check http://www.iana.org/assignments/protocol-numbers/ for
 *  updated info */
bool IPv4Header::validateNextProto(u8 p){

    if ( p <= 140 )
        return true;
    else
        return false;

} /* End of validateNextProto() */



bool IPv4Header::validateNextProto(){

  return validateNextProto( this->getNextProto() );

} /* End of validateNextProto() */




/** Computes the IPv4 header checksum and sets the ip_sum field to the right
 *  value. */
int IPv4Header::setSum(){

  h.ip_sum = 0;

  /* ip_checksum() comes from libdnet */
  ip_checksum((void*)&h, 20 + ipoptlen );

  return OP_SUCCESS;

} /* End of setSum() */


/** @warning Sum is set to supplied value with NO byte ordering conversion
 *  performed.
 *  @warning If sum is supplied this way, no error checks are made. Caller is
 *  responsible for the correctness of the value. */
int IPv4Header::setSum(u16 s){

  h.ip_sum = s;

  return OP_SUCCESS;
} /* End of setSum() */


/** Set the checksum field to a random value */
int IPv4Header::setSumRandom(){

   h.ip_sum=get_random_u16();

  return OP_SUCCESS;
} /* End of setRandomSum() */



/** Returns the value of the checksum field.
 *  @warning The returned value is in NETWORK byte order, no conversion is
 *  performed */
u16 IPv4Header::getSum(){

  return h.ip_sum;

} /* End of getSum() */


/** Returns the value of the checksum field in HOST byte order */
u16 IPv4Header::getSumHBO(){

  return ntohs(h.ip_sum);

} /* End of getSumHBO() */




/** Sum must be in NETWORK byte order */
bool IPv4Header::validateSum(u8 *pkt, int len, u16 sum){


/* TODO: Unimplemented!! */
/* Check validateSum(void) when implementing this. */

        return true;

} /* End of validateNextProto() */



bool IPv4Header::validateSum(){

  /* TODO: Test this! */
  return validateSum( (u8*)&h, 20+ipoptlen, getSum() );

        return true;

} /* End of validateNextProto() */






/** Sets destination IP address.
 *  @warning Destination IP must be supplied in NETWORK byte order. Usually
 *  all regular library functions return IPs in network byte order so there
 *  should be no need to worry. If you have an ip in host byte order use
 *  method setDstIPHBO() */
int IPv4Header::setDstIP(u32 d){

  h.ip_dst.s_addr =  d;

  return OP_SUCCESS;

} /* End of getDstIP() */


/** Sets destination IP address.
 *  @warning Destination IP must be supplied in HOST byte order. Byte ordering
 *  conversion is performed using htonl(). If you have an ip in network byte
 *  order use method setDstIP() */
int IPv4Header::setDstIPHBO(u32 d){

  h.ip_dst.s_addr =  htonl(d);

  return OP_SUCCESS;

} /* End of setDstIPHBO() */



int IPv4Header::setDstIP(char *i){

  if( !strcasecmp(i, "rand") || !strcasecmp(i, "random") ){
    h.ip_dst.s_addr = get_random_u32();
    return OP_SUCCESS;
  }

  u32 aux;

  if ( atoIP(i, &aux) != OP_SUCCESS ){
    printf("setSrcIP(): Couldn't set source IP\n");
    return OP_FAILURE;

   }
   else{
        setDstIP(aux);
    }

  return OP_SUCCESS;
} /* End of setDestination() */


/** Returns destination ip
 *  @warning Returned value is in NETWORK byte order. */
u32 IPv4Header::getDstIP(){

  return h.ip_dst.s_addr;

} /* End of getDstIP() */



/** Returns destination ip
 *  @warning Returned value is in HOST byte order. */
u32 IPv4Header::getDstIPHBO(){

  return ntohl(h.ip_dst.s_addr);

} /* End of getDstIPHBO() */


/** A four byte number always represents a valid IP address. Even 0.0.0.0 and
 *  255.255.255.255 can be valid under some circumstances so there is not much
 *  we can do here testing the IP */
bool IPv4Header::validateDstIP(u32 i){

    return true;
}
bool IPv4Header::validateDstIP(){

    return validateDstIP( this->getDstIP() );
}







/** Sets source IP address.
 *  @warning Destination IP must be supplied in NETWORK byte order. Usually
 *  all regular library functions return IPs in network byte order so there
 *  should be no need to worry. If you have an ip in host byte order use
 *  method setSrcIPHBO() */
int IPv4Header::setSrcIP(u32 d){

  h.ip_src.s_addr =  d;

  return OP_SUCCESS;

} /* End of getSrcIP() */


/** Sets source IP address.
 *  @warning Destination IP must be supplied in HOST byte order. Byte ordering
 *  conversion is performed using htonl(). If you have an ip in network byte
 *  order use method setSrcIP() */
int IPv4Header::setSrcIPHBO(u32 d){

  h.ip_src.s_addr =  htonl(d);

  return OP_SUCCESS;

} /* End of setSrcIPHBO() */



int IPv4Header::setSrcIP(char *i){

  if( !strcasecmp(i, "rand") || !strcasecmp(i, "random") ){
    h.ip_src.s_addr = get_random_u32();
    return OP_SUCCESS;
  }

  u32 aux;

  if ( atoIP(i, &aux) != OP_SUCCESS ){
    printf("setSrcIP(): Couldn't set source IP\n");
    return OP_FAILURE;

   }
   else{
        setSrcIP(aux);
    }

  return OP_SUCCESS;
} /* End of setDestination() */


/** Returns source ip
 *  @warning Returned value is in NETWORK byte order. */
u32 IPv4Header::getSrcIP(){

  return h.ip_src.s_addr;

} /* End of getSrcIP() */



/** Returns source ip
 *  @warning Returned value is in HOST byte order. */
u32 IPv4Header::getSrcIPHBO(){

  return ntohl(h.ip_src.s_addr);

} /* End of getSrcIPHBO() */



/** A four byte number always represents a valid IP address. Even 0.0.0.0 and
 *  255.255.255.255 can be valid under some circumstances so there is not much
 *  we can do here testing the IP */
bool IPv4Header::validateSrcIP(u32 i){

    return true;
}
bool IPv4Header::validateSrcIP(){

    return validateSrcIP( this->getSrcIP() );
}





int IPv4Header::setOpts(char *txt){

 int foo=0;
 int bar=0;
 int ret=0;
 u8 buffer[128];

 if(txt==NULL){
    printf("setOpts(): NULL pointer supplied.\n");
    return OP_FAILURE;
 }

 /* Parse IP options */
 ret =  parse_ip_options(txt, buffer, 128, &foo, &bar);
 /* TODO: Do we need to use the firsthopoff and lasthopoff values? (4th and 5th
  * parameter) */


 if( ret < 0 )
    return OP_FAILURE;
 else{

     /* Copy options to our IP header */
     memcpy(h.options, buffer, ret);
     this->ipoptlen=ret;
     this->length += ret;

     this->setHeaderLength();

    return OP_SUCCESS;
 }
} /* End of setOpts() */





u8 *IPv4Header::getOpts(){

  return h.options;

} /* End of getOpts() */


u8 *IPv4Header::getOpts(int *len){

  if(len==NULL)
    printf("getOpts(): NULL pointer supplied.\n");

  *len=ipoptlen;
  return h.options;

} /* End of getOpts() */



int IPv4Header::printOptions(){

    char *p;
    p=print_ip_options(this->h.options, this->ipoptlen);
    printf("%s", p);

    return OP_SUCCESS;
} /* End of printOptions() */

char *IPv4Header::getOptionsString(){

    char *p;
    p=print_ip_options(this->h.options, this->ipoptlen);

    return p;
} /* End of getOptionsString() */




bool IPv4Header::validateOptions(u8 *buff, int len){

   if(buff==NULL || len<0 || len>40 ){
        //error!
    }

   if(len == 0 )
        return true;




  /* TODO: unimplemented! */
    return true;
}


bool IPv4Header::validateOptions(){

    return validateOptions( h.options, ipoptlen );

    return true;
}



int IPv4Header::printHeader(){

 printf("VER:%d ", getVersion() );
 printf("HLEN:%d ", getHeaderLength() );
 printf("TOS:%d ", getTOS() );
 printf("TLEN:%d ", getTotalLength() );
 printf("ID:%X ", getIdentification() );
 printf("OFF:%d ", getFragOffset() );
 printf("Flags:( %s%s%s) ",
        (getRF()==true) ? "RF ": "",
        (getDF()==true) ? "DF ": "",
        (getMF()==true) ? "MF ": ""
      );
 printf("TTL:%d ", getTTL() );
 printf("PROT:%d ", getNextProto() );
 printf("CSUM:%X ", getSum() );
 printf("SRC:%s ", IPtoa( getSrcIP() ) );
 printf("DST:%s ", IPtoa( getDstIP() ) );

 if ( ipoptlen>0)
    printf("OPTS:%s ", getOptionsString() );

  printf("\n");

  return OP_SUCCESS;

} /* End of printHeader() */
