
/***************************************************************************
 * IPv4Header.h -- The IPv4Header Class represents an IPv4 datagram. It    *
 * contains methods to set any header field. In general, these methods do  *
 * error checkings and byte order conversion.                              *
 *                                                                         *
 ***********************IMPORTANT NMAP LICENSE TERMS************************
 *                                                                         *
 * The Nmap Security Scanner is (C) 1996-2009 Insecure.Com LLC. Nmap is    *
 * also a registered trademark of Insecure.Com LLC.  This program is free  *
 * software; you may redistribute and/or modify it under the terms of the  *
 * GNU General Public License as published by the Free Software            *
 * Foundation; Version 2 with the clarifications and exceptions described  *
 * below.  This guarantees your right to use, modify, and redistribute     *
 * this software under certain conditions.  If you wish to embed Nmap      *
 * technology into proprietary software, we sell alternative licenses      *
 * (contact sales@insecure.com).  Dozens of software vendors already       *
 * license Nmap technology such as host discovery, port scanning, OS       *
 * detection, and version detection.                                       *
 *                                                                         *
 * Note that the GPL places important restrictions on "derived works", yet *
 * it does not provide a detailed definition of that term.  To avoid       *
 * misunderstandings, we consider an application to constitute a           *
 * "derivative work" for the purpose of this license if it does any of the *
 * following:                                                              *
 * o Integrates source code from Nmap                                      *
 * o Reads or includes Nmap copyrighted data files, such as                *
 *   nmap-os-db or nmap-service-probes.                                    *
 * o Executes Nmap and parses the results (as opposed to typical shell or  *
 *   execution-menu apps, which simply display raw Nmap output and so are  *
 *   not derivative works.)                                                *
 * o Integrates/includes/aggregates Nmap into a proprietary executable     *
 *   installer, such as those produced by InstallShield.                   *
 * o Links to a library or executes a program that does any of the above   *
 *                                                                         *
 * The term "Nmap" should be taken to also include any portions or derived *
 * works of Nmap.  This list is not exclusive, but is meant to clarify our *
 * interpretation of derived works with some common examples.  Our         *
 * interpretation applies only to Nmap--we don't speak for other people's  *
 * GPL works.                                                              *
 *                                                                         *
 * If you have any questions about the GPL licensing restrictions on using *
 * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
 * we also offer alternative license to integrate Nmap into proprietary    *
 * applications and appliances.  These contracts have been sold to dozens  *
 * of software vendors, and generally include a perpetual license as well  *
 * as providing for priority support and updates as well as helping to     *
 * fund the continued development of Nmap technology.  Please email        *
 * sales@insecure.com for further information.                             *
 *                                                                         *
 * As a special exception to the GPL terms, Insecure.Com LLC grants        *
 * permission to link the code of this program with any version of the     *
 * OpenSSL library which is distributed under a license identical to that  *
 * listed in the included COPYING.OpenSSL file, and distribute linked      *
 * combinations including the two. You must obey the GNU GPL in all        *
 * respects for all of the code used other than OpenSSL.  If you modify    *
 * this file, you may extend this exception to your version of the file,   *
 * but you are not obligated to do so.                                     *
 *                                                                         *
 * If you received these files with a written license agreement or         *
 * contract stating terms other than the terms above, then that            *
 * alternative license agreement takes precedence over these comments.     *
 *                                                                         *
 * Source is provided to this software because we believe users have a     *
 * right to know exactly what a program is going to do before they run it. *
 * This also allows you to audit the software for security holes (none     *
 * have been found so far).                                                *
 *                                                                         *
 * Source code also allows you to port Nmap to new platforms, fix bugs,    *
 * and add new features.  You are highly encouraged to send your changes   *
 * to nmap-dev@insecure.org for possible incorporation into the main       *
 * distribution.  By sending these changes to Fyodor or one of the         *
 * Insecure.Org development mailing lists, it is assumed that you are      *
 * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
 * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
 * will always be available Open Source, but this is important because the *
 * inability to relicense code has caused devastating problems for other   *
 * Free Software projects (such as KDE and NASM).  We also occasionally    *
 * relicense the code to third parties as discussed above.  If you wish to *
 * specify special license conditions of your contributions, just say so   *
 * when you send them.                                                     *
 *                                                                         *
 * This program is distributed in the hope that it will be useful, but     *
 * WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
 * General Public License v2.0 for more details at                         *
 * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
 * included with Nmap.                                                     *
 *                                                                         *
 ***************************************************************************
 * The code in this file was originally written by Luis MartinGarcia       *
 * during Google Summer of Code programme, 2009.                           *
 *                                                                         *
 ***************************************************************************/
#ifndef IPV4HEADER_H
#define IPV4HEADER_H 1

#include "NetworkLayerElement.h"

#define DEFAULT_IP_TTL 64

/// class IPv4Header -
class IPv4Header : public NetworkLayerElement {


    private:

    public:

        struct my_ipv4
        {
        #if WORDS_BIGENDIAN
            u8 ip_v:4;                     /* Version                        */
            u8 ip_hl:4;                    /* Header length                  */
        #else
            u8 ip_hl:4;                    /* Header length                  */
            u8 ip_v:4;                     /* Version                        */
        #endif
            u8 ip_tos;                     /* Type of service                */
            u16 ip_len;                    /* Total length                   */
            u16 ip_id;                     /* Identification                 */
            u16 ip_off;                    /* Fragment offset field          */
        #define IP_RF 0x8000               /* Reserved fragment flag         */
        #define IP_DF 0x4000               /* Dont fragment flag             */
        #define IP_MF 0x2000               /* More fragments flag            */
        #define IP_OFFMASK 0x1fff          /* Mask for fragmenting bits      */
            u8 ip_ttl;                     /* Time to live                   */
            u8 ip_p;                       /* Protocol                       */
            u16 ip_sum;                    /* Checksum                       */
            struct in_addr ip_src;         /* Source IP address              */
            struct in_addr ip_dst;         /* Destination IP address         */

        #define MAX_IP_OPTIONS_LEN 40
            u8 options[MAX_IP_OPTIONS_LEN];  /* IP Options                   */
        /** From wikipedia entry for IPv4: "[...]the IP header size (20 bytes
         *  minimum; 60 bytes maximum)"
         *  So 60-20 that leaves 40bytes max options length. */
        }h;


    int ipoptlen; /**< Length of IP options */


    /* Misc */
    IPv4Header();
    ~IPv4Header();
    void zero();
    u8 *getBufferPointer();

    /* IP version */
    int setVersion();
    u8 getVersion();
    bool validateVersion();
    bool validateVersion(u8 ver);

    /* Header Length */
    int setHeaderLength();
    int setHeaderLength(u8 l);
    u8 getHeaderLength();
    bool validateHeaderLength();
    bool validateHeaderLength(u8 len);

    /* Type of Service */
    int setTOS(u8 v);
    u8 getTOS();
    bool validateTOS(u8 t);
    bool validateTOS();

    /* Total lenght of the datagram */
    int setTotalLength();
    int setTotalLength(u16 l);
    u16 getTotalLength();
    bool validateTotalLength(u16 i);
    bool validateTotalLength();

    /* Identification value */
    int setIdentification();
    int setIdentification(u16 i);
    u16 getIdentification();
    bool validateIdentification(u16 i);
    bool validateIdentification();

    /* Fragment Offset */
    int setFragOffset();
    int setFragOffset(u16 f);
    u16 getFragOffset();
    bool validateFragOffset(u16 f);
    bool validateFragOffset();


    /* Flags */
    int setRF();
    int unsetRF();
    bool getRF();
    bool validateRF();

    int setDF();
    int unsetDF();
    bool getDF();

    int setMF();
    int unsetMF();
    bool getMF();

    bool validateFlags();

    /* Time to live */
    int setTTL();
    int setTTL(u8 t);
    u8 getTTL();

    /* Next protocol */
    int setNextProto(u8 p);
    int setNextProto(const char *p);
    u8 getNextProto();
    bool validateNextProto(u8 p);
    bool validateNextProto();


    /* Checksum */
    int setSum();
    int setSum(u16 s);
    int setSumRandom();
    u16 getSum();
    u16 getSumHBO();
    bool validateSum(u8 *pkt, int len, u16 sum);
    bool validateSum();

    /* Destination IP */
    int setDstIP(char *i);
    int setDstIP(u32 d);
    int setDstIPHBO(u32 d);
    u32 getDstIP();
    u32 getDstIPHBO();
    bool validateDstIP(u32 i);
    bool validateDstIP();


    /* Source IP */
    int setSrcIP(char *i);
    int setSrcIP(u32 d);
    int setSrcIPHBO(u32 d);
    u32 getSrcIP();
    u32 getSrcIPHBO();
    bool validateSrcIP(u32 i);
    bool validateSrcIP();

    /* IP Options */
    int setOpts(char *txt);
    u8 *getOpts();
    u8 *getOpts(int *len);
    int printOptions();
    char *getOptionsString();
    bool validateOptions(u8 *buff, int len);
    bool validateOptions();

    int printHeader();

}; /* End of class IPv4Header */



#endif
