#!/bin/bash
NPING_VERSION=`grep '^\#[ \t]*define[ \t]\+NPING_VERSION' ../nping.h | sed -e 's/.*"\(.*\)".*/\1/' -e 'q'`
SRCS=`grep '^\export SRCS = ' ../Makefile.in | sed "s/export SRCS = //g" | sed "s/@COMPAT_SRCS@//g"`
HDRS=`grep '^\export HDRS = ' ../Makefile.in | sed "s/export HDRS = //g"` 
OBJS=`grep '^\OBJS = ' ../Makefile.in | sed "s/OBJS = //g" | sed "s/@COMPAT_OBJS@//g"`
DOCS2DIST=`grep '^\export DOCS2DIST = ' ../Makefile.in | sed "s/export DOCS2DIST = //g"`
MISC2DIST=`grep '^\export MISC2DIST = ' ../Makefile.in | sed "s/export MISC2DIST = //g"`
RELEASE_DATE=`date +"%b %d, %Y"`
echo "NPING_VERSION=$NPING_VERSION"
echo "SRCS=$SRCS"
echo "HDRS=$HDRS"
echo "OBJS=$OBJS"
echo "DOCS2DIST=$DOCS2DIST"
echo "MISC2DIST=$MISC2DIST"
echo "RELEASE_DATE=$RELEASE_DATE"

# Now we should be in nmap-exp/luis/nping/nping-priv/
NPINGCURRENTDIR=`pwd`

# Determine absolute path to tarball directory
cd $NPINGCURRENTDIR
cd ../tarballs
NPINGTARBALLDIR=`pwd`
echo "NPINGTARBALLDIR=$NPINGTARBALLDIR"
cd $NPINGCURRENTDIR

# Determine absolute path to windows installer directory
cd $NPINGCURRENTDIR
cd ../mswin32/nsis
NPINGWININSTDIR=`pwd`
echo "NPINGWININSTDIR=$NPINGWININSTDIR"
cd $NPINGCURRENTDIR

# Determine absolute path to web directory
cd $NPINGCURRENTDIR
cd ../../../../web/nmap/nping/
NPINGWEBDIR=`pwd`
echo "NPINGWEBDIR=$NPINGWEBDIR"
cd $NPINGCURRENTDIR

# Determine absolute path to docs directory
cd $NPINGCURRENTDIR
cd ../docs
NPINGDOCSDIR=`pwd`
echo "NPINGDOCSDIR=$NPINGDOCSDIR"
cd $NPINGCURRENTDIR

# Determine absolute path to nping main dir
cd $NPINGCURRENTDIR
cd ..
NPINGDIR=`pwd`
echo "NPINGDIR=$NPINGDIR"
cd $NPINGCURRENTDIR

# Determine absolute path to libpcap dir
cd $NPINGCURRENTDIR
cd ../libpcap
LIBPCAPDIR=`pwd`
echo "LIBPCAPDIR=$LIBPCAPDIR"
cd $NPINGCURRENTDIR

# Determine absolute path to libdnet dir
cd $NPINGCURRENTDIR
cd ../libdnet-stripped
LIBDNETDIR=`pwd`
echo "LIBDNETDIR=$LIBDNETDIR"
cd $NPINGCURRENTDIR

# Determine absolute path to nsock dir
cd $NPINGCURRENTDIR
cd ../nsock
NSOCKDIR=`pwd`
echo "NSOCKDIR=$NSOCKDIR"
cd $NPINGCURRENTDIR

# Determine absolute path to nbase dir
cd $NPINGCURRENTDIR
cd ../nbase
NBASEDIR=`pwd`
echo "NBASEDIR=$NBASEDIR"
cd $NPINGCURRENTDIR

# Determine absolute path to windows stuff dir
cd $NPINGCURRENTDIR
cd ../mswin32
WIN32STUFFDIR=`pwd`
echo "WIN32STUFFDIR=$WIN32STUFFDIR"
cd $NPINGCURRENTDIR

# Determine absolute path to windows installer directory
cd $NPINGCURRENTDIR
cd ../mswin32/nsis
NPINGWININSTDIR=`pwd`
echo "NPINGWININSTDIR=$NPINGWININSTDIR"
cd $NPINGCURRENTDIR

# Determine absolute path to Mac OS X stuff dir
cd $NPINGCURRENTDIR
cd ../macosx
MACOSXSTUFFDIR=`pwd`
echo "MACOSXSTUFFDIR=$MACOSXSTUFFDIR"
cd $NPINGCURRENTDIR

echo "Current working nping-priv/ directory is = $NPINGCURRENTDIR"
read -p "Would you like to continue with the process? [Y/N]"
if  [ "$REPLY" == "Y" ]; then
    echo "Great. Starting..."
else
    echo "See in another life, brother..."
    exit
fi


echo "[+] Compiling the latest version"
cd $NPINGDIR
make

echo "[+] Running the man page generator"
cd $NPINGDOCSDIR
./genmanpage.sh

echo "[+] Removing unnecessary files"
cd $NPINGDOCSDIR
rm -f ./config.cache
make clean
make -C $LIBPCAPDIR clean
make -C $LIBDNETDIR clean
rm -rf nping-$NPING_VERSION


echo "[+] Creating new temp directory for the new release"
cd $NPINGDIR
mkdir nping-$NPING_VERSION
mkdir nping-$NPING_VERSION/docs


echo "[+] Copying the neccessary files to the new temp dir"
cd $NPINGDIR
cp $SRCS ./nping-$NPING_VERSION/
cp $HDRS ./nping-$NPING_VERSION/
cp $MISC2DIST ./nping-$NPING_VERSION/

cd $NPINGDOCSDIR
cp $DOCS2DIST ../nping-$NPING_VERSION/docs/

cd $NPINGDIR
cp -R $LIBPCAPDIR ./nping-$NPING_VERSION/ 
cp -R $LIBDNETDIR ./nping-$NPING_VERSION/
cp -R $NBASEDIR ./nping-$NPING_VERSION/ 
cp -R $NSOCKDIR ./nping-$NPING_VERSION/
cp -R $WIN32STUFFDIR ./nping-$NPING_VERSION/



echo "[+] Removing SVN directories"
cd $NPINGDIR/nping-$NPING_VERSION 
TEMP1=`pwd`
read -p "All SVN dirs in $TEMP1 are going to be removed. Would you like to continue with the process? [Y/N]"
if  [ "$REPLY" == "Y" ]; then
    rm -rf `find . -type d -name .svn`
else
    echo "Quitting..."
    exit
fi




echo "[+] Creating a temp dir for the tarballs"
cd $NPINGDIR
rm -rf tarballs
mkdir tarballs




#echo "[+] Generating BZIP tarball"
#cd $NPINGDIR
#tar cjf tarballs/nping-$NPING_VERSION.tar.bz2 nping-$NPING_VERSION


echo "[+] Generating GZIP tarball"
cd $NPINGDIR
tar czf tarballs/nping-$NPING_VERSION.tar.gz nping-$NPING_VERSION


echo "[+] Generating MD5 and SHA256 checksums"
echo "[+] Generating MD5 and SHA256 checksums for tarballs..."
cd tarballs 
md5sum nping-$NPING_VERSION.tar.gz > nping-$NPING_VERSION.tar.gz.md5
sha256sum nping-$NPING_VERSION.tar.gz > nping-$NPING_VERSION.tar.gz.sha256
#cd tarballs && md5sum nping-$NPING_VERSION.tar.bz2 > md5sum nping-$NPING_VERSION.tar.bz2.md5
#cd tarballs && sha256sum nping-$NPING_VERSION.tar.bz2 > md5sum nping-$NPING_VERSION.tar.bz2.sha256

echo "[+] Generating MD5 and SHA256 checksums for windows installer..."
cd $NPINGWININSTDIR
md5sum nping-$NPING_VERSION-setup.exe > nping-$NPING_VERSION-setup.exe.md5
sha256sum nping-$NPING_VERSION-setup.exe > nping-$NPING_VERSION-setup.exe.sha256


echo "[+] Generating MD5 and SHA256 checksums for MAC OS X disk image..."
cd $MACOSXSTUFFDIR
md5sum nping-$NPING_VERSION.dmg > nping-$NPING_VERSION.dmg.md5
sha256sum nping-$NPING_VERSION.dmg > nping-$NPING_VERSION.dmg.sha256

echo "[+] Removing temp nping-$NPING_VERSION directory"
rm -rf nping-$NPING_VERSION
echo "[+] Tarballs generated successfully!"
